package Ravada::Front::Domain;

use warnings;
use strict;

=head2 NAME

Ravada::Front::Domain - Frontent domain information for Ravada

=cut

use Carp qw(cluck confess croak);
use Data::Dumper;
use JSON::XS;
use Moose;

use Ravada::Front::Domain::KVM;
use Ravada::Front::Domain::Void;

no if $] >= 5.020000, warnings => "experimental::signatures";
use if $] >= 5.020000, feature => qw(signatures);

with 'Ravada::Domain';

###########################################################################
#
has 'readonly' => (
    isa => 'Int'
    ,is => 'ro'
    ,default => 1
);

our $CONNECTOR = \$Ravada::Front::CONNECTOR;
#
###########################################################################

sub BUILD {
    my $self = shift;
    my $arg = shift;

    my $id = $arg->{id};
    my $name = $arg->{name};

    $self->_select_domain_db( id => $id)    if defined $id;
    $self->_select_domain_db( name => $name)    if defined $name;

    $self->{_data}->{id} = $id      if defined $id;
    $self->{_data}->{name} = $name  if defined $name;

#    confess "ERROR: Domain '".$self->name." not found "
#        if $self->is_volatile && ! $self->is_active;
}

sub open {
    my $self = shift;
    my $id = shift;

    my $domain = Ravada::Front::Domain->new( id => $id );
    if ($domain->type eq 'KVM') {
        $domain = Ravada::Front::Domain::KVM->new( id => $id );
    } elsif ($domain->type eq 'Void') {
        $domain = Ravada::Front::Domain::Void->new( id => $id );
    }
    die "ERROR: Unknown domain id: $id\n"
        unless exists $domain->{_data}->{name} && $domain->{_data}->{name};
    return $domain;
}

sub autostart {
    my $self = shift;

    return $self->_data('autostart')
}
sub _do_force_shutdown  { confess "TODO" }
sub add_volume          { confess "TODO" }
sub clean_swap_volumes  { confess "TODO" }
sub disk_device         { confess "TODO" }
sub disk_size           { confess "TODO" }

sub display {
    my $self = shift;
    my $user = shift;

    return $self->_data('display');
}

sub force_shutdown      { confess "TODO" }

sub get_info {
    my $self = shift;

     my $info = $self->_data('info');
     return {} if !$info;
     return decode_json($info);
}
sub hybernate           { confess "TODO" }
sub hibernate           { confess "TODO" }

sub internal_id {
    my $self = shift;

    return $self->_data('internal_id')
}

sub is_active {
    my $self = shift;

    return 1 if $self->_data('status') eq 'active';
    return 0;
}

sub is_volatile_clones {
    my $self = shift;

    return $self->_data('volatile_clones')
}

sub is_hibernated {
    my $self = shift;

    return 1 if $self->_data('status') eq 'hibernated';
    return 0;
}

sub is_paused {
    my $self = shift;

    return 1 if $self->_data('status') eq 'paused';
    return 0;
}

sub is_removed          { return 0 }
sub list_volumes        { confess "TODO" }

sub name {
    my $self = shift;

    return $self->{_data}->{name}   if exists $self->{_data} && $self->{_data}->{name};
    return $self->_data('name')
}

sub pause               { confess "TODO" }
sub prepare_base        { confess "TODO" }
sub remove              { confess "TODO" }
sub rename              { confess "TODO" }
sub resume              { confess "TODO" }
sub screenshot          { confess "TODO" }

sub set_max_mem         { confess "TODO" }
sub set_memory          { confess "TODO" }
sub shutdown            { confess "TODO" }
sub shutdown_now        { confess "TODO" }
sub spinoff_volumes     { confess "TODO" }
sub start               { confess "TODO" }

sub get_driver {}
sub get_controller_by_name { }
sub list_controllers {}

sub set_controller {}
sub remove_controller {}
1;
